/*
 * ECGenerator.java
 *
 * Created on 4. duben 2002, 11:22
 */

package SOFA.Connector.ECG;

import org.w3c.dom.*;
import SOFA.SOFAnode.Made.TIR.*;
import SOFA.SOFAnode.Made.TIR.Access.TIRAccessMethods;

/**
 *
 * @author  ghort
 * @version
 */
public class ECGenerator {
	
	protected java.util.LinkedList plugin; /* of ECGPluginInterface */
	
	protected javax.xml.parsers.DocumentBuilderFactory documentBuilderFactory;
	protected javax.xml.parsers.DocumentBuilder documentBuilder;
	
	protected Document configDoc;

	protected SOFA.Connector.EEG.EEGenerator eeGenerator;

	static public String getTypesPackage() {
		return "SOFA.Connector.ECG.Types";
	}
		
	/** Creates new EEGenerator */
	public ECGenerator() throws ECGeneratorException {
		try {
			eeGenerator=new SOFA.Connector.EEG.EEGenerator();	
		} catch (SOFA.Connector.EEG.EEGeneratorException e) {
			throw new ECGeneratorException("Can't initialize EEGenerator.",e);
		}
		
		try {
			documentBuilderFactory=javax.xml.parsers.DocumentBuilderFactory.newInstance();
			documentBuilder=documentBuilderFactory.newDocumentBuilder();
			
			configDoc=documentBuilder.parse(new java.io.File(getECGConfigRoot()+"/"+System.getProperty("SOFA.Connector.ECG.Config","conf.xml")));
			
			Element conParent=configDoc.getDocumentElement();
			Node conPlugins, conPlugin, conProp;
			
			plugin=new java.util.LinkedList();
			
			System.out.println("Loading ECG plugins:");
			conPlugins=conParent.getFirstChild();
			while (conPlugins!=null) {
				if (conPlugins.getNodeType()==Node.ELEMENT_NODE && conPlugins.getNodeName().equals("ECGPlugins")) {
					conPlugin=conPlugins.getFirstChild();
					while (conPlugin!=null) {
						if (conPlugin.getNodeType()==Node.ELEMENT_NODE && conPlugin.getNodeName().equals("ECGPlugin")) {
							
							java.util.HashMap props=new java.util.HashMap();
							conProp=conPlugin.getFirstChild();
							while (conProp!=null) {
								if (conProp.getNodeType()==Node.ELEMENT_NODE) {
									Element el=(Element)conProp;
									if (el.getTagName().equals("property") && el.hasAttribute("name") && el.hasAttribute("value")) {
										props.put(el.getAttribute("name"),el.getAttribute("value"));
									}
								}
								conProp=conProp.getNextSibling();
							}
							System.out.println("  Loading plugin ("+((Element)conPlugin).getAttribute("class")+"):");
							
							Class cls=Class.forName(((Element)conPlugin).getAttribute("class"));
							Class[] coPar=new Class[2];
							coPar[0]=Class.forName("SOFA.Connector.EEG.EEGenerator");
							coPar[1]=Class.forName("java.util.HashMap");
							java.lang.reflect.Constructor co=cls.getConstructor(coPar);
							
							Object[] coInst=new Object[2];
							coInst[0]=eeGenerator;
							coInst[1]=props;
							ECGPluginInterface plgObject=(ECGPluginInterface)co.newInstance(coInst);
							
							plugin.add(plgObject);
							System.out.println("  Plugin "+plgObject.getName()+" ("+((Element)conPlugin).getAttribute("class")+") loaded.");
						}
						conPlugin=conPlugin.getNextSibling();
					}
				}
				conPlugins=conPlugins.getNextSibling();
			}
			
		} catch (Exception e) {
			throw new ECGeneratorException("Can't parse configuration",e);
		}
	}
	
	public ConnectorsTechnologyDescriptor canGenerate(ConnectorQuery query) {
		java.util.Iterator iter;
		ECGPluginInterface plgObject;
		ConnectorsTechnologyDescriptor ctDescr=new ConnectorsTechnologyDescriptor();
		java.util.LinkedList tDescr=new java.util.LinkedList();
		
		ctDescr.type=query.type;
		ctDescr.instanceName=query.instanceName;
		iter=plugin.iterator();
		try {
			for (;;) {
				plgObject=(ECGPluginInterface)iter.next();
				tDescr.addAll(java.util.Arrays.asList(plgObject.canGenerate(query)));
			}
		} catch (java.util.NoSuchElementException e) {
		}

		int i;
		Object arr[];
		ctDescr.technologies=new ConnectorTechnologyDescriptor[(arr=tDescr.toArray()).length];
		for (i=0;i<arr.length;i++) {
			ctDescr.technologies[i]=(ConnectorTechnologyDescriptor)arr[i];
		}

		return ctDescr;
	}
	
	public ConnectorOutputDescriptor generate(ConnectorInputDescriptor conDescr) throws ECGeneratorException {
		java.util.Iterator iter;
		ECGPluginInterface plgObject;
		
		int colonIndex=conDescr.implementation.indexOf(':');
		String pluginName;
		
		if (colonIndex==-1) {
			pluginName=conDescr.implementation;
		} else {
			pluginName=conDescr.implementation.substring(0,colonIndex);
		}
		
                SOFA.Connector.Property[] newProps = new SOFA.Connector.Property [conDescr.props.length + 1];
                for (int i=0; i<conDescr.props.length; i++) {
                  newProps[i] = conDescr.props[i];
                }
		String resolverFName = null;
                try {
                  resolverFName = java.io.File.createTempFile("conGen", ".table").getAbsolutePath();
                  newProps[conDescr.props.length] = new SOFA.Connector.Property("resolverFile", resolverFName);
                } catch (java.io.IOException e) {
                  throw new ECGeneratorException ("Can't create temp file.", e);
                }
                conDescr.props = newProps;
                
		System.out.println("Generating connector '"+conDescr.implementation+"':");

		iter=plugin.iterator();
		try {
			for (;;) {
				plgObject=(ECGPluginInterface)iter.next();
				if (pluginName.equals(plgObject.getName())) {
					try {
						ConnectorOutputDescriptor ret;
						ret=plgObject.generate(conDescr);
						System.out.println("Connector generated.");
                                                System.out.print("Augmenting classes of this connector...");
						ResolverInfo info = prepareResolverInfo(resolverFName);
						callResolver(info);
                                                System.out.println("OK");
						return ret;
					} catch (Exception e) {
						throw new ECGeneratorException("Plugin failed",e);
					}
				}
			}
		} catch (java.util.NoSuchElementException e) {
			throw new ECGeneratorException("Can't generate element.",e);
		}
	}

	static public String getECGRoot() throws ECGeneratorException {
		try {
			return System.getProperty("SOFA.Connector.Root","SOFA/Connector")+"/ECG";
		} catch (Exception e) {
			throw new ECGeneratorException("Can't access property SOFA.Connector.Root",e);
		}
	}

	static public String getECGConfigRoot() throws ECGeneratorException {
		String configDir;
		try {
			configDir=System.getProperty("SOFA.Connector.Config");
		} catch (Exception e) {
			throw new ECGeneratorException("Can't access property SOFA.Connector.Config",e);
		}
		if (configDir==null) {
			configDir=getECGRoot();
		} else {
			configDir+="/ECG";
		}
		return configDir;
	}

  private class ResolverInfo {
    java.util.Hashtable table;
    java.util.ArrayList classFiles;
    public ResolverInfo() {
      table = new java.util.Hashtable ();
      classFiles = new java.util.ArrayList ();
    }
  }

  /** Prepares information for resolver.
    * 
    * @param fname    name with type and class file info
    * @return         resolver info
    * @throws java.io.IOException  io exception
    */
  private ResolverInfo prepareResolverInfo(String fname) throws java.io.IOException, java.rmi.RemoteException, SOFA.Connector.EEG.CodeWriter.CodeWriterException, SOFA.SOFAnode.Made.TIR.TIRExceptLock {
    ResolverInfo info = new ResolverInfo();
    java.io.BufferedReader fr = new java.io.BufferedReader(new java.io.FileReader(fname));
    String line = null;
    while ((line = fr.readLine()) != null) {
      java.util.ArrayList ifaces = new java.util.ArrayList ();
      String classFile = null;
      java.util.StringTokenizer st = new java.util.StringTokenizer(line);
      while (st.hasMoreTokens()) {
        if (classFile != null)
          ifaces.add(classFile);
        classFile = st.nextToken();
      }
      for (int i=0; i<ifaces.size(); i++) {
        InterfaceDef rIface = (InterfaceDef) TIRAccessMethods.lookupCDLContained(SOFA.Connector.TIRAccess.TIRAccess.repository, (String) ifaces.get(i));
        performCDLType(rIface, info.table);
        Contained[] op = TIRAccessMethods.getOperAttrOfInterface(SOFA.Connector.TIRAccess.TIRAccess.repository, (String) ifaces.get(i));
        for (int j=0; j<op.length; j++) {
          if (op[j] instanceof AttributeDef) {
            performCDLType(((AttributeDef) op[j]).type(), info.table);
          } else { // it is OperationDef
            performCDLType(((OperationDef) op[j]).result(), info.table);
            ExceptionDef[] exc = ((OperationDef) op[j]).exceptions();
            for (int k=0; k<exc.length; k++) {
              performCDLType(exc[k], info.table);
            }
            ParamDescr[] par = ((OperationDef) op[j]).params();
            for (int k=0; k<par.length; k++) {
              performCDLType(par[k].type(), info.table);
            }
          }
        }
      }
      info.classFiles.add(classFile);
    }
    fr.close();
    return info;
  }

  private void performCDLType(CDLType type, java.util.Hashtable table) throws java.rmi.RemoteException, SOFA.Connector.EEG.CodeWriter.CodeWriterException {
    if (type.get_def_kind().value() != DefinitionKind.dk_Primitive) {
      String javaType = SOFA.Connector.EEG.CodeWriter.CDL2JavaMapping.getTypeName(type, ParamMode.PARAM_IN);
      String javaTypeHolder = SOFA.Connector.EEG.CodeWriter.CDL2JavaMapping.getTypeName(type, ParamMode.PARAM_OUT);
      String version = ((Contained) type).get_identification().version();
      table.put(SOFA.Tools.Resolver.Resolver.classNameToVMClassName(javaType), SOFA.Tools.Resolver.Resolver.cdlClassNameToVMClassName(javaType, version));
      table.put(SOFA.Tools.Resolver.Resolver.classNameToVMClassName(javaTypeHolder), SOFA.Tools.Resolver.Resolver.cdlClassNameToVMClassName(javaTypeHolder, version));
    }
    
  }

  private void callResolver(ResolverInfo info) throws java.io.IOException {
    SOFA.Tools.Resolver.Resolver resolver = new SOFA.Tools.Resolver.Resolver (info.table);
    for(int i=0; i<info.classFiles.size(); i++) {
      java.io.FileInputStream is = new java.io.FileInputStream((String) info.classFiles.get(i));
      byte[] b = resolver.process(is);
      is.close();
      java.io.FileOutputStream os = new java.io.FileOutputStream ((String) info.classFiles.get(i));
      os.write(b);
      os.close();
    }
  }
}
