/*
 * JImplWriterDirectorBase.java
 *
 * Created on 13. duben 2002, 21:08
 */

package SOFA.Connector.EEG.EEM.Actions;

import SOFA.SOFAnode.Made.TIR.*;
import SOFA.SOFAnode.Made.TIR.Access.TIRAccessMethods;
import SOFA.Connector.EEG.CodeWriter.*;

/**
 *
 * @author  ghort
 * @version 
 */
public class JImplWriterDirectorBase implements JWriterDirector {
	protected SOFA.Connector.Property[] params;
	protected String provIfaceCDL;

	/** Creates new JImplWriterDirectorBase */
	public JImplWriterDirectorBase(String provIfaceCDL, SOFA.Connector.Property[] params) {
		this.provIfaceCDL=provIfaceCDL;
		this.params=params;
	}

	public JImplWriterDirectorBase(SOFA.Connector.Property[] params) {
		this.params=params;
	}

	protected String targetCall(OperationDef method, String params) throws JWriterDirectorException {
		try {
			StringBuffer ret=new StringBuffer();
			ret.append("this.target."+method.get_identification().name()+"(");
			ret.append(params);
			ret.append(")");
			return ret.toString();
		} catch (java.rmi.RemoteException e) {
			throw new JWriterDirectorException("Cannot access TIR.",e);
		}
	}
	
	protected String declareRet(OperationDef method, String retName) throws JWriterDirectorException {
		try {
			CDLType retType=method.result();
			if (!(retType.get_def_kind().value()==DefinitionKind.dk_Primitive && 
				((PrimitiveDef)retType).kind().value()==PrimitiveKind.pk_void)) {
				return CDL2JavaMapping.getTypeName(retType,ParamMode.PARAM_IN)+" "+retName+";";
			}
			return "";
		} catch (java.rmi.RemoteException e) {
			throw new JWriterDirectorException("Cannot access TIR.",e);
		} catch (CodeWriterException e) {
			throw new JWriterDirectorException("Cannot map type.",e);
		}
	}

	protected String assignRet(OperationDef method, String retName) throws JWriterDirectorException {
		try {
			CDLType retType=method.result();
			if (!(retType.get_def_kind().value()==DefinitionKind.dk_Primitive && 
				((PrimitiveDef)retType).kind().value()==PrimitiveKind.pk_void)) {
				return retName+"=";
			}
			return "";
		} catch (java.rmi.RemoteException e) {
			throw new JWriterDirectorException("Cannot access TIR.",e);
		}
	}

	protected String returnRet(OperationDef method, String retName) throws JWriterDirectorException {
		try {
			CDLType retType=method.result();
			if (!(retType.get_def_kind().value()==DefinitionKind.dk_Primitive && 
				((PrimitiveDef)retType).kind().value()==PrimitiveKind.pk_void)) {
				return "return "+retName+";";
			}
			return "return;";
		} catch (java.rmi.RemoteException e) {
			throw new JWriterDirectorException("Cannot access TIR.",e);
		}
	}

	protected String managerTieImpl() {
		return "\tprotected SOFA.Connector.EEG.ConnectorManagerInterface manager;\n"+
			"\n"+
			"\tpublic void setManagerTie(SOFA.Connector.EEG.ConnectorManagerInterface manager) {\n"+
			"\t\tthis.manager=manager;\n"+
			"\t}\n"+
			"\n"+
			"\tpublic SOFA.Connector.EEG.ConnectorManagerInterface getManagerTie() {\n"+
			"\t\treturn manager;\n"+
			"\t}\n";
	}

	protected String linkImpl() {
		String reqIface=SOFA.Connector.Property.findFirst(params,"reqIfaceMapped");
		return "\tprotected "+reqIface+" target;\n"+
			"\n"+
			"\tpublic void link(Object target) {\n"+
			"\t\tthis.target=("+reqIface+")target;\n"+
			"\t}\n";
	}

	public String[] getImplements() throws JWriterDirectorException {
		return null;
	}
	 
	public String getClassBody() throws JWriterDirectorException {
		return linkImpl()+"\n"+managerTieImpl();
	}
	 
	public String getExtends() throws JWriterDirectorException {
		return null;
	}
	 
	public String getMethodDef(OperationDef mt) throws JWriterDirectorException {
		return JIfaceNormal.getMethodDef(mt);
	}

	public String getMethodBody(OperationDef method)  throws JWriterDirectorException {
		return 
			"\t\t"+declareRet(method,"ret")+"\n"+
			"\t\t"+assignRet(method,"ret")+targetCall(method,JIfaceNormal.targetCallParams(method))+";\n"+
			"\t\t"+returnRet(method,"ret");
	}

	public boolean defaultImplements() {
		return true;
	}
}
