/* $Id: CompBindOper.java,v 1.1.1.1 2003/02/11 16:19:40 bures Exp $ */
package SOFA.SOFAnode.Made.CDL;
import SOFA.SOFAnode.Made.TIR.*;
import java.rmi.RemoteException;
 
public class CompBindOper extends CompObjectIm {
  public static final int b_provreq = 0;
  public static final int b_sub = 1;

  int kind;
  List elements;
  List subs;
  int provreq;   // if it refer prov or req

  CompArchitecture arch;

  public CompBindOper(List els, CompArchitecture a) {
    super(ObjectsKind.o_BindOper);
    kind = b_provreq;
    elements = els;
    subs = null;
    arch = a;
  }

  public CompBindOper(List sub, List els, CompArchitecture a) {
    super(ObjectsKind.o_BindOper);
    kind = b_sub;
    elements = els;
    subs = sub;
    arch = a;
  }

  public String toString() {
    StringBuffer sb = new StringBuffer();
    CompBindType type;
    if (kind == b_sub) {
      subs.toFirst();
      type = (CompBindType) subs.aktual();
      sb.append(type.name);
      if (type.exprs.size() != 0) {
        sb.append("[");
        type.exprs.toFirst();
        for (int i=0;i<type.exprs.size();i++) {
          if (i!=0)
            sb.append(",");
          sb.append(((CompExprOper) type.exprs.aktual()).toString());
        }
        sb.append("]");
      }
      sb.append(".");
    }
    elements.toFirst();
    type = (CompBindType) elements.aktual();
    sb.append(type.name);
    if (type.exprs.size() != 0) {
      sb.append("[");
      type.exprs.toFirst();
      for (int i=0;i<type.exprs.size();i++) {
        if (i!=0)
          sb.append(",");
        sb.append(((CompExprOper) type.exprs.aktual()).toString());
      }
      sb.append("]");
    }
    return sb.toString();
  }

  public void checkConsist(EnumList props, CompRepository rep) throws CDLExceptCheck, CDLExceptLock, CDLExceptRemote {
    int i;
    elements.toFirst();
    for (i=0;i<elements.size();i++) {
      ((CompBindType) elements.aktual()).checkConsist(props,rep);
      elements.toNext();
    }
    if (kind == b_sub) {
      subs.toFirst();
      for (i=0;i<subs.size();i++) {
        ((CompBindType) subs.aktual()).checkConsist(props,rep);
        subs.toNext();
      }
    }
  }

  public BindOperDef toNormal(ArchitectureDef arch, CDLRepository normRep) throws CDLExceptToNormal, CDLExceptRemote, CDLExceptLock {
    try {
      if (kind == b_provreq) {
        BindOperProvReq ret = arch.create_bindoperprovreq();
        elements.toFirst();
        for (int i=0; i<elements.size(); i++) {
          CompBindType akt = (CompBindType) elements.aktual();
          if (akt.exprs.size() == 0) {
            if (ret.add_bindtypenormal(akt.name)==null)
              throw new CDLExceptToNormal("Can't create bind type normal");
          } else {
            ExprOperDef[] ind = new ExprOperDef [akt.exprs.size()];
            akt.exprs.toFirst();
            for(int j=0;j<akt.exprs.size();j++) {
              ind[j] = ((CompExprOper) akt.exprs.aktual()).toNormal(normRep);
              akt.exprs.toNext();
            }
            if (ret.add_bindtypearray(akt.name, ind)==null)
              throw new CDLExceptToNormal("Can't create bind type array");
          }
          elements.toNext();
        }
        return ret;
      } else {
        BindOperSub ret = arch.create_bindopersub();
        elements.toFirst();
        for (int i=0; i<elements.size(); i++) {
          CompBindType akt = (CompBindType) elements.aktual();
          if (akt.exprs.size() == 0) {
            if (ret.add_bindtypenormal(akt.name)==null)
              throw new CDLExceptToNormal("Can't create bind type normal");
          } else {
            ExprOperDef[] ind = new ExprOperDef [akt.exprs.size()];
            akt.exprs.toFirst();
            for(int j=0;j<akt.exprs.size();j++) {
              ind[j] = ((CompExprOper) akt.exprs.aktual()).toNormal(normRep);
              akt.exprs.toNext();
            }
            if (ret.add_bindtypearray(akt.name, ind)==null)
              throw new CDLExceptToNormal("Can't create bind type array");
          }
          elements.toNext();
        }
        subs.toFirst();
        for (int i=0; i<subs.size(); i++) {
          CompBindType akt = (CompBindType) subs.aktual();
          if (akt.exprs.size() == 0) {
            if (ret.add_subnormal(akt.name)==null)
              throw new CDLExceptToNormal("Can't create bind type normal");
          } else {
            ExprOperDef[] ind = new ExprOperDef [akt.exprs.size()];
            akt.exprs.toFirst();
            for(int j=0;j<akt.exprs.size();j++) {
              ind[j] = ((CompExprOper) akt.exprs.aktual()).toNormal(normRep);
              akt.exprs.toNext();
            }
            if (ret.add_subarray(akt.name, ind)==null)
              throw new CDLExceptToNormal("Can't create bind type array");
          }
          subs.toNext();
        }
        return ret;
      }
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    } catch (TIRExceptCreate ecr) {
      throw new CDLExceptToNormal("Can't create bind");
    }
  }

  // prov/req
  public boolean testBOPR() throws CDLExceptRemote, CDLExceptLock {
    int i,j;
    elements.toFirst();
    if (arch.frame.isNew) {
      CompContainer con = arch.rep;
      FullID id = arch.frame.ref;
      id.sn.names.toFirst();
      for(i=0;i<id.sn.names.size();i++) {
        CompContained[] a = con.lookup((String) id.sn.names.aktual());
        if (a==null || a.length==0) 
          return false;
        if (a.length==1) {
          if (a[0].isContainer()) con = (CompContainer) a[0];
          else return false;
        } else {
          return false;
        }
        id.sn.names.toNext();
      }
      CompContained frame = con.lookup(id.name, id.version);
      if (frame==null)
        return false;
      CompContained sec = ((CompContainer) frame).lookup(((CompBindType)elements.aktual()).name, id.version);
      if (sec == null)  // don't exist prov/req with this name
        return false;
      if (sec.objectKind()!=ObjectsKind.o_Provides && sec.objectKind()!=ObjectsKind.o_Requires)
        return false;  // it isn't prov/req
      provreq = sec.objectKind();
      String tc = null;
      if (sec.objectKind()==ObjectsKind.o_Provides) {
        tc = ((CompProvide) sec).typeCode;
      } else {
        tc = ((CompRequire) sec).typeCode;
      }
      int k=0;
      for (i=0;i<elements.size();i++) {
        for (j=0;j<((CompBindType) elements.aktual()).exprs.size();j++) {
          if (tc.charAt(k)!='a')
            return false;
          k++;
        }
        elements.toNext();
      }
      if (tc.charAt(k)!='i')
        return false;
      return true;
    } else {  // search in normal rep
      try {
        Container con = arch.rep.cdlRepository;
        FullID id = arch.frame.ref;
        id.sn.names.toFirst();
        for(i=0;i<id.sn.names.size();i++) {
          Contained[] a = con.lookup_name((String) id.sn.names.aktual());
          if (a==null || a.length==0) 
            return false;
          if (a.length==1) {
            if (a[0] instanceof Container) con = (Container) a[0];
            else return false;
          } else {
            boolean found = false;
            for(j=0;j<a.length;j++) {
              if (a[j].get_identification().version().compareTo(id.version)==0) {
                if ( !(a[j] instanceof Container)) return false;
                found = true;
                con = (Container) a[j];
                break;
              }
            }
            if (!found) return false;
          }
          id.sn.names.toNext();
        }
        Contained frame = con.lookup(id.name, id.version);
        if (frame==null)
          return false;
        Contained sec = ((Container) frame).lookup(((CompBindType)elements.aktual()).name, id.version);
        if (sec == null)  // don't exist prov/req with this name
          return false;
        if (sec.get_def_kind().value()!=DefinitionKind.dk_Provides && sec.get_def_kind().value()!=DefinitionKind.dk_Requires)
          return false;  // it isn't prov/req
        provreq = sec.get_def_kind().value();
        StringBuffer tc = new StringBuffer();
        boolean ok = false;
        CDLType tp;
        if (sec.get_def_kind().value()==DefinitionKind.dk_Provides) {
          tp = ((ProvideDef) sec).provide();
        } else {
          tp = ((RequireDef) sec).require();
        }
        while (!ok) {
          switch (tp.get_def_kind().value()) {
          case DefinitionKind.dk_Interface:
            tc.append('i');
            ok = true;
            break;
          case DefinitionKind.dk_Array:
            tp = ((ArrayDef) tp).element_type();
            tc.append('a');
            break;
          case DefinitionKind.dk_Typedef:
            tp = ((TypedefDef) tp).original_type();
            break;
          }
        }
        int k=0;
        for (i=0;i<elements.size();i++) {
          for (j=0;j<((CompBindType) elements.aktual()).exprs.size();j++) {
            if (tc.charAt(k)!='a')
              return false;
            k++;
          }
          elements.toNext();
        }
        if (tc.charAt(k)!='i')
          return false;
        return true;
      } catch (TIRExceptLock e) {
        throw new CDLExceptLock("Repository is locked.");
      } catch (RemoteException e) {
        throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
      }
    }
  }

  // subs
  public boolean testBOS() {
    int i,j;
    subs.toFirst();
    CompContained inst = arch.lookup(((CompBindType) subs.aktual()).name, arch.getIdent().version);
    if (inst==null)
      return false;
    if (inst.objectKind()!=ObjectsKind.o_Inst)
      return false;
    String tc = ((CompInst) inst).typeCode;
    int k=0;
    for (i=0;i<subs.size();i++) {
      for (j=0;j<((CompBindType) subs.aktual()).exprs.size();j++) {
        if (tc.charAt(k)!='a')
          return false;
        k++;
      }
      elements.toNext();
    }
    if (tc.charAt(k)!='f')
      return false;
    return true;
  }
}
