/* $Id: CompFrame.java,v 1.1.1.1 2003/02/11 16:19:40 bures Exp $ */
package SOFA.SOFAnode.Made.CDL;
import SOFA.SOFAnode.Made.TIR.*;
import java.rmi.RemoteException;

public class CompFrame extends CompContainedContainerIm implements CompType {
  CompProtOper protocol;
  EnumList propNames;
  public boolean isSystem;
  boolean consistChecked;

  public CompFrame(ID id, CompContainer in, CompRepository inRep, boolean sys) {
    super(ObjectsKind.o_Frame, id, in, inRep);
    protocol = null;
    propNames = new EnumList();
    isSystem = sys;
    consistChecked = false;
//    Debug.message("Vytvarim frame: "+id);
  }

  public boolean addObject(CompContained o) {
    o.getIdent().version = id.version;
    if (o.objectKind() == ObjectsKind.o_Property) {
      propNames.addName(o.getIdent().name);
    }
    return super.addObject(o);
  }

  public static EnumList getPropNames(CompReffer ref, CompRepository rep) throws CDLExceptRemote, CDLExceptLock {
    int i,j;
    if (ref.isNew) {
      CompContainer con = rep;
      ref.ref.sn.names.toFirst();
      for(i=0;i<ref.ref.sn.names.size();i++) {
        CompContained[] a = con.lookup((String) ref.ref.sn.names.aktual());
        if (a==null || a.length==0) 
          return null;
        if (a.length==1) {
           if (a[0].isContainer()) con = (CompContainer) a[0];
             else return null;
        } else {
          return null;
        }
        ref.ref.sn.names.toNext(); 
      }
      CompContained sec = con.lookup(ref.ref.name, ref.ref.version);
      if (sec!=null && sec.objectKind()==ObjectsKind.o_Frame)
        return ((CompFrame) sec).propNames;
      else
        return null;
    } else {
      try {
        Container con = rep.cdlRepository;
        ref.ref.sn.names.toFirst();
        for(i=0;i<ref.ref.sn.names.size();i++) {
          Contained[] a = con.lookup_name((String) ref.ref.sn.names.aktual());
          if (a==null || a.length==0) 
            return null;
          if (a.length==1) {
            if (a[0] instanceof Container) con = (Container) a[0];
            else return null;
          } else {
            boolean found = false;
            for(j=0;j<a.length;j++) {
              if (a[j].get_identification().version().compareTo(ref.ref.version)==0) {
                if ( !(a[j] instanceof Container)) return null;
                found = true;
                con = (Container) a[j];
                break;
              }
            }
            if (!found) return null;
          }
          ref.ref.sn.names.toNext();
        }
        Contained sec = con.lookup(ref.ref.name, ref.ref.version);
        if (sec!=null && sec.get_def_kind().value() == DefinitionKind.dk_Frame ) {
          DefinitionKind dkprop = rep.workRepository.get_spec_def_kind(DefinitionKind.dk_Property);
          Contained[] prop = ((FrameDef) sec).contents(dkprop);
          EnumList ret = new EnumList();
          for (i=0;i<prop.length;i++) {
            ret.addName(prop[i].get_identification().name());
          }
          return ret;
        } else {
          return null;
        }
      } catch (TIRExceptLock e) {
        throw new CDLExceptLock("Repository is locked.");
      } catch (RemoteException e) {
        throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
      }
      
    }
  }

  public void checkConsist(EnumList props, CompRepository rep) throws CDLExceptCheck, CDLExceptLock, CDLExceptRemote {
    int i;
    CompContained[] cont = contents();
    for (i=0;i<cont.length;i++) {
      cont[i].checkConsist(propNames, rep);
    }

    if (rep.useProtocols) {
      if (protocol == null && contents().length!=0)
        throw new CDLExceptCheck("No protocol for frame "+fullName());
    }
    if (!consistChecked) {
      if (rep.testProtocols) {
        String res;
        for (i=0;i<cont.length;i++) {
          if (cont[i].objectKind() == ObjectsKind.o_Provides) {
  	    CompProvide p = (CompProvide) cont[i];
            Output.out.println("Checking protocol of provides "+p.getIdent().name+" of frame: "+getIdent().name);
            String[] operations = p.operations.toArray();
            prependStringArray("?"+p.getIdent().name+".", operations);
            try {
              String ifaceProt = p.ifaceProtocol();
              if (ifaceProt.compareTo("")==0) {
                continue;
              }
              if (System.getProperty("cdl.debug","no").compareTo("yes")==0) {
                res = SOFA.SOFAnode.Util.ProtocolChecker.int2frame(ifaceProt, true, operations, protocol.toText());
              }
              long tmb = java.util.Calendar.getInstance().getTime().getTime();
              res = repository().protocolChecker.int2frame(ifaceProt, true, operations, protocol.toText());
              long tme = java.util.Calendar.getInstance().getTime().getTime();
              Output.out.println("Time of protocol checking of provides: "+getIdent().name+"."+p.getIdent().name+": "+Long.toString(tme-tmb)+" ms");
              if (res != null)
                Output.out.println("Warning: "+res+" ("+fullName()+")");
            } catch (SOFA.SOFAnode.Util.SyntaxErrorException e) {
              Output.out.println("Protocol syntax error\n"+e.getMessage());
            }
  	  }
          if (cont[i].objectKind() == ObjectsKind.o_Requires) {
  	    CompRequire r = (CompRequire) cont[i];
            Output.out.println("Checking protocol of requires "+r.getIdent().name+" of frame: "+getIdent().name);
            String[] operations = r.operations.toArray();
            prependStringArray("!"+r.getIdent().name+".", operations);
            try {
              String ifaceProt = r.ifaceProtocol();
              if (ifaceProt.compareTo("")==0) {
                continue;
              }
              if (System.getProperty("cdl.debug","no").compareTo("yes")==0) {
                res = SOFA.SOFAnode.Util.ProtocolChecker.int2frame(ifaceProt, false, operations, protocol.toText());
              }
              long tmb = java.util.Calendar.getInstance().getTime().getTime();
              res = repository().protocolChecker.int2frame(ifaceProt, false, operations, protocol.toText());
              long tme = java.util.Calendar.getInstance().getTime().getTime();
              Output.out.println("Time of protocol checking of requires: "+getIdent().name+"."+r.getIdent().name+": "+Long.toString(tme-tmb)+" ms");
              if (res != null)
  	        Output.out.println("Warning: "+res+" ("+fullName()+")");
            } catch (SOFA.SOFAnode.Util.SyntaxErrorException e) {
              Output.out.println("Protocol syntax error\n"+e.getMessage());
            }
  	  }
        }
        System.gc();
      }
    }
    consistChecked = true;
  }

  public void addToNormal(Container obj, CDLRepository normRep, java.util.ArrayList delayed) throws CDLExceptToNormal, CDLExceptRemote, CDLExceptLock {
    if (id.isin == IDKind.versiontag) {
      try {
        ((Contained)obj).tag(id.tag);
      } catch (RemoteException e) {
        throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
      } catch (TIRExceptLock e) {
        throw new CDLExceptLock("Repository is locked.");
      } catch (TIRExceptCreate ecr) {
        throw new CDLExceptToNormal("Can't set tag for "+ fullName());
      }
    }
    cont.toFirst();
    try {
      for (int i=0;i<cont.size(); i++) {
        CompContained akt = (CompContained) cont.aktual();
        switch (akt.objectKind()) {
        // *****************
        case ObjectsKind.o_Property:
          try {
            PropertyDef prp = ((FrameDef) obj).create_property(akt.getIdent().name,((CompProperty)akt).tp.toNormal(normRep) );
            if (prp == null)
              throw new CDLExceptToNormal("Can't create property "+((CompContained)akt).fullName());
          } catch (TIRExceptCreate ecr) {
            throw new CDLExceptToNormal("Can't create property "+((CompContained)akt).fullName()+"\n"+
                                         ecr.getMessage() );
          }
          break;
        // *****************
        case ObjectsKind.o_Provides:
          try {
            ProvideDef prov = ((FrameDef) obj).create_provide(akt.getIdent().name,((CompProvide)akt).type.toNormal(normRep) );
            if (prov == null)
              throw new CDLExceptToNormal("Can't create provide "+((CompContained)akt).fullName());
          } catch (TIRExceptCreate ecr) {
            throw new CDLExceptToNormal("Can't create provide "+((CompContained)akt).fullName());
          }
          break;
        // *****************
        case ObjectsKind.o_Requires:
          try {
            RequireDef req = ((FrameDef) obj).create_require(akt.getIdent().name,((CompRequire)akt).type.toNormal(normRep) );
            if (req == null)
              throw new CDLExceptToNormal("Can't create require "+((CompContained)akt).fullName());
          } catch (TIRExceptCreate ecr) {
            throw new CDLExceptToNormal("Can't create require "+((CompContained)akt).fullName());
          }
          break;
        default:
          throw new CDLExceptToNormal("Unexcepted kind of object"); 
        }
        cont.toNext();
      }
      if (protocol != null) {
        try {
          ProtocolDef newProt = ((FrameDef) obj).create_protocol();
          ProtocolOperDef p = CompProtOper.toNormalFrame(protocol, newProt);
	  newProt.protocol(p);
	} catch (TIRExceptCreate ecr) {
	  throw new CDLExceptToNormal("Can't create protocol in "+ fullName()+"\n"+
                                         ecr.getMessage());
	}
      }
     
    } catch (RemoteException e) {
      throw new CDLExceptRemote("Remote exception occured: "+e.getMessage());
    } catch (TIRExceptLock e) {
      throw new CDLExceptLock("Repository is locked.");
    }
  }

  /** from CompType */
  public CDLType toNormal(CDLRepository newRep) throws CDLExceptToNormal {
    // never call this
    throw new CDLExceptToNormal("It should never throw");
  }

  public boolean isProvReqOperation(String req, String opername) {
    cont.toFirst();
    for (int i=0; i<cont.size(); i++) {
      if (((CompContained) cont.aktual()).getIdent().isNameEqual(req)) {
        if (((CompObject)cont.aktual()).objectKind() == ObjectsKind.o_Requires) {
          return  ((CompRequire) cont.aktual()).isOperation(opername);
        } else {
          if (((CompObject)cont.aktual()).objectKind() == ObjectsKind.o_Provides) {
            return ((CompProvide) cont.aktual()).isOperation(opername); 
	  }
	}
        break;
      }
      cont.toNext();
    }
    return false;
  }

  public String protocolToText() {
    if (protocol != null) {
      return protocol.toText();
    }
    return "-- no protocol --";
  }
  
  /** prepend string "what" to all strings in "a" */
  static void prependStringArray(String what, String[] a) {
    for (int i=0;i<a.length;i++) {
      a[i] = what + a[i];
    }
  }
}
