/* $Id: ObjectList.java,v 1.1.1.1 2003/02/11 16:19:41 bures Exp $ */
package SOFA.SOFAnode.Made.CodeGen;

import SOFA.SOFAnode.Made.TIR.*;
 
/** Collection of repository objects.
  * 
  * @author Petr Hnetynka
  */
class ObjectList extends java.util.ArrayList {
   /** Returns the object at the specified position in this list.
     * 
     * @exception IndexOutOfBoundsException     if index out of range (index < 0 || index >= size()).
     */
   public Contained getObject(int index) {
     return (Contained) get(index);
   }

   /** Returns specified object.
     * 
     * @param absName  absolute name of object whose presence in this List is to be tested.
     * @param ver      version identifier of object whose presence in this List is to be tested.
     * @exception RemoteException  remote exception
     */
   public Contained getObject(String absName, String ver) throws java.rmi.RemoteException {
     Contained obj;
     for (int i=0;i<size();i++) {
       if ((obj = getObject(i)).get_identification().is_equal(absName, ver))
         return obj;
     }
     return null;
   }

   /** Returns <tt>true</tt> if this list contains the specified object.
     *
     * @param absName  absolute name of object whose presence in this List is to be tested.
     * @param ver      version identifier of object whose presence in this List is to be tested.
     * @exception RemoteException  remote exception 
     */
   public boolean containsObject(String absName, String ver) throws java.rmi.RemoteException {
     for (int i=0;i<size();i++) {
       if (getObject(i).get_identification().is_equal(absName, ver))
         return true;
     }
     return false;
   }

   /** Returns <tt>true</tt> if this list contains the specified object.
     *
     * @param obj      repository object whose presence in this List is to be tested.
     * @exception RemoteException  remote exception 
     */
   public boolean containsObject(Contained obj) throws java.rmi.RemoteException {
     Identification idl = obj.get_identification();
     for (int i=0;i<size();i++) {
       if (getObject(i).get_identification().is_equal(idl.absolute_name().name(), idl.version()))
         return true;
     }
     return false;
   }

   /** Adds object to list only if this object already is not in list (uses containsObject()).
     *
     * @param   obj   added object 
     * @exception RemoteException  remote exception 
     */
   public void addObject(Contained obj) throws java.rmi.RemoteException {
     Identification idl = obj.get_identification();
     Contained inObj;
     if ((inObj=getObject(idl.absolute_name().name(), idl.version()))==null) {
       add(obj);
     }/* else {
       remove(indexOfObject(idl.absolute_name().name(), idl.version()));
       add(obj);
     }*/
   }

   /** Returns index of the specified object if this list contains it.
     *
     * @param absName  absolute name of object whose presence in this List is to be tested.
     * @param ver      version identifier of object whose presence in this List is to be tested.
     * @exception RemoteException  remote exception
     * @return the index of the the argument in this list; returns -1 if the object is not found.
     */
   public int indexOfObject(String absName, String ver) throws java.rmi.RemoteException {
     for (int i=0;i<size();i++) {
       if (getObject(i).get_identification().is_equal(absName, ver))
         return i;
     }
     return -1;
   }
   
   /** Merge this lists together.
     *
     * @param obj list with new objects, which are added to this list
     * @exception RemoteException  remote exception
     */
   public void mergeTogether(ObjectList obj) throws java.rmi.RemoteException {
     if (obj!=null) {
       for (int i=0; i<obj.size();i++) {
         addObject(obj.getObject(i));
       }
     }
   }
}
