/* $Id: BindDefImpl.java,v 1.1.1.1 2003/02/11 16:19:41 bures Exp $ */
package SOFA.SOFAnode.Made.TIR.Impl;
import SOFA.SOFAnode.Made.TIR.*;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.io.*;

public class BindDefImpl extends ContainedImpl implements BindDef {
  DefinitionKindImpl defKindImpl;
  StateKindImpl stKindImpl;
  BindOperDef rhs;
  BindOperDef lhs;
  BindMode mod;
  String connector;

  BindDefImpl normal;

  public BindDefImpl(Identification ident, Container in, Repository inrep, BindOperDef l, BindOperDef r, BindMode m, String using) throws RemoteException {
    super(ident,in,inrep);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Bind);
    stKindImpl = new StateKindImpl(StateKind.sk_work);
    rhs = r;
    lhs = l;
    mod = m;
    connector = using;
    normal = null;
  }

  public BindDefImpl(Container in, Repository inrep) throws RemoteException {
    super(null ,in,inrep);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Bind);
    stKindImpl = new StateKindImpl(StateKind.sk_normal);
    rhs = null;
    lhs = null;
    mod = null;
    connector = null;
    normal = null;
  }

  public BindDefImpl(BindDefImpl a, Container in, Repository inrep) throws RemoteException {
    super(a.id ,in,inrep);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Bind);
    stKindImpl = new StateKindImpl(StateKind.sk_work);
    rhs = a.rhs;
    lhs = a.lhs;
    mod = a.mod;
    connector = a.connector;
    normal = a;
  }

  /* from interface TIRObject */
  public DefinitionKind get_def_kind() throws RemoteException {
    return (DefinitionKind) defKindImpl;
  }

  /* from interface TIRObject */
  public StateKind get_state() throws RemoteException {
    return (StateKind) stKindImpl;
  }

  public BindOperDef lhsop() throws RemoteException {
    return lhs;
  }
  
  public BindOperDef rhsop() throws RemoteException {
    return rhs;
  }
  
  public BindMode mode() throws RemoteException {
    return mod;
  }

  public String using() throws RemoteException {
    return connector;
  }

  public void save(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      ((IdentificationImpl)id).save(st.curOutFile);
      ((TIRImplObject) rhs).save(st);
      ((TIRImplObject) lhs).save(st);
      ((TIRImplObject) mod).save(st);
      Storage.writeString(st.curOutFile, connector);
    } catch (IOException e) {
      throw new TIRExceptStorage("Can't write in file "+st.currentFile+".");
    }
  }

  public void load(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      id = new IdentificationImpl();
      ((IdentificationImpl)id).load(st.curInFile);
      int kind = st.curInFile.readInt();
      switch (kind) {
        case BindOperKind.bok_provreq: 
	  rhs = new BindOperProvReqImpl(StateKind.sk_normal);
	  break;
        case BindOperKind.bok_sub: 
	  rhs = new BindOperSubImpl(StateKind.sk_normal);
	  break;
      }
      ((TIRImplObject) rhs).load(st);
      kind = st.curInFile.readInt();
      switch (kind) {
        case BindOperKind.bok_provreq: 
	  lhs = new BindOperProvReqImpl(StateKind.sk_normal);
	  break;
        case BindOperKind.bok_sub: 
	  lhs = new BindOperSubImpl(StateKind.sk_normal);
	  break;
      }
      ((TIRImplObject) lhs).load(st);
      mod = new BindModeImpl();
      ((TIRImplObject) mod).load(st);
      connector = Storage.readString(st.curInFile);
    } catch (IOException e) {
      throw new TIRExceptStorage("Can't write in file "+st.currentFile+".");
    }
  }

  public void postLoad(RepositoryImpl r) throws RemoteException, TIRExceptStorage {
    ((TIRImplObject) rhs).postLoad(r);
    ((TIRImplObject) lhs).postLoad(r);
  }

  public boolean isNew() {
    return ((stKindImpl.value()==StateKind.sk_work) && normal==null);
  }

  public void canCommit() throws RemoteException, TIRExceptCommit {;}

  public void doCommit(Container in, Repository rep) throws RemoteException {
    if (stKindImpl.value()==StateKind.sk_work) {
      stKindImpl.toNormal();
      rhs = bo_toNormal(rhs);
      lhs = bo_toNormal(lhs);
      ((TIRImplObject)rhs).doCommit(in,rep);
      ((TIRImplObject)lhs).doCommit(in,rep);
    }
  }

  public void doAbort(long workId) throws RemoteException {
    ((TIRImplObject)rhs).doAbort(workId);
    ((TIRImplObject)lhs).doAbort(workId);
  }

  private BindOperDef bo_toNormal(BindOperDef b) throws RemoteException {
    if (b != null) {
      switch (b.get_bo_kind().value()) {
      case BindOperKind.bok_provreq: {
        BindOperProvReqImpl ret = new BindOperProvReqImpl(StateKind.sk_work);
        BindType[] el = ((BindOperProvReq) b).elements();
        for (int i=0;i<el.length;i++) {
          switch (el[i].get_bt_kind().value()) {
          case BindTypeKind.btk_normal:
            ret.add(new BindTypeNormalImpl(new String(((BindTypeNormal) el[i]).name())));
            break;
          case BindTypeKind.btk_array:
             ret.add(new BindTypeArrayImpl(new String(((BindTypeArray) el[i]).name()), ((BindTypeArray) el[i]).array_indexes()));
            break;           
          }
        }
        return ret;
      }  
      case BindOperKind.bok_sub: {
        BindOperSubImpl ret = new BindOperSubImpl(StateKind.sk_work);
        BindType[] el = ((BindOperSub) b).elements();
        for (int i=0;i<el.length;i++) {
          switch (el[i].get_bt_kind().value()) {
          case BindTypeKind.btk_normal:
            ret.addEl(new BindTypeNormalImpl(new String(((BindTypeNormal) el[i]).name())));
            break;
          case BindTypeKind.btk_array:
             ret.addEl(new BindTypeArrayImpl(new String(((BindTypeArray) el[i]).name()), ((BindTypeArray) el[i]).array_indexes()));
            break;           
          }
        }
        el = ((BindOperSub) b).subs();
        for (int i=0;i<el.length;i++) {
          switch (el[i].get_bt_kind().value()) {
          case BindTypeKind.btk_normal:
            ret.addSub(new BindTypeNormalImpl(new String(((BindTypeNormal) el[i]).name())));
            break;
          case BindTypeKind.btk_array:
             ret.addSub(new BindTypeArrayImpl(new String(((BindTypeArray) el[i]).name()), ((BindTypeArray) el[i]).array_indexes()));
            break;           
          }
        }
        return ret;
        }
      } // -- switch
    }
    return null;
  }
}
