/* $Id: BindOperProvReqImpl.java,v 1.1.1.1 2003/02/11 16:19:41 bures Exp $ */
package SOFA.SOFAnode.Made.TIR.Impl;
import SOFA.SOFAnode.Made.TIR.*;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.io.*;

public class BindOperProvReqImpl extends UnicastRemoteObject implements BindOperProvReq, TIRImplObject {
  DefinitionKindImpl defKindImpl;
  StateKindImpl stKindImpl;
  BindOperKind bok;
  int numOfEl;
  LiItem firstEl;
  LiItem lastEl;

  public BindOperProvReqImpl(int state) throws RemoteException {
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_BindOper);
    stKindImpl = new StateKindImpl(state);
    bok = new BindOperKindImpl(BindOperKind.bok_provreq);
    numOfEl = 0;
    firstEl = lastEl = null;
  }

  /* from interface TIRObject */
  public DefinitionKind get_def_kind() throws RemoteException {
    return (DefinitionKind) defKindImpl;
  }

  /* from interface TIRObject */
  public StateKind get_state() throws RemoteException {
    return (StateKind) stKindImpl;
  }

  public BindOperKind get_bo_kind() throws RemoteException {
    return bok;
  }

  public BindType[] elements() throws RemoteException {
    LiItem l = firstEl;
    BindType[] ret = new BindType [numOfEl];
    for(int i = 0;i<numOfEl;i++) {
      ret[i] = (BindType) l.obj;
      l = l.next;
    }
    return ret;
  }

  public void save(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      st.curOutFile.writeInt(bok.value());
      st.curOutFile.writeInt(numOfEl);
      LiItem l= firstEl;
      for(int i=0;i<numOfEl;i++) {
        ((TIRImplObject) l.obj).save(st);
        l = l.next;
      }  
    } catch (IOException e) {
      throw new TIRExceptStorage("Can't write in directory "+st.current+".");
    }   
  }

  public void load(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      int len = st.curInFile.readInt();
      for(int i=0;i<len;i++) {
        BindType akt;
        switch (st.curInFile.readInt()) {
        case BindTypeKind.btk_normal:
          akt = new BindTypeNormalImpl();
          break;
        case BindTypeKind.btk_array:
          akt = new BindTypeArrayImpl();
          break;
        default:
          throw new TIRExceptStorage("Unexpected kind of object in \""+st.currentFile+"\".");
        }
        ((TIRImplObject) akt).load(st);
        add(akt);
      }
    } catch (IOException e) {
      throw new TIRExceptStorage("Can't read in directory "+st.current+".");
    }
  }

  public void add(BindType what) {
    LiItem item = new LiItem(what);
    if (firstEl == null) {
      firstEl = lastEl = item;
    } else {
      lastEl.next = item;
      item.prev = lastEl;
      lastEl = item;
    }
    numOfEl++;
  }

  public void postLoad(RepositoryImpl r) throws RemoteException, TIRExceptStorage {
    LiItem l = firstEl;
    for(int i=0;i<numOfEl;i++) {
      ((TIRImplObject) l.obj).postLoad(r);
      l = l.next;
    }
  }

  public boolean isNew() {
    return (stKindImpl.value()==StateKind.sk_work);
  }

  public void canCommit() throws RemoteException, TIRExceptCommit {
    if (numOfEl == 0)
      throw new TIRExceptCommit("Empty prov/req bind");
    LiItem l = firstEl;
    for(int i=0;i<numOfEl;i++) {
      ((TIRImplObject) l.obj).canCommit();
      l = l.next;
    }
  }

  public void doCommit(Container in, Repository rep) throws RemoteException {
    if (stKindImpl.value()==StateKind.sk_normal)
      return;
    if (isNew()) {
      LiItem l = firstEl;
      for(int i=0;i<numOfEl;i++) {
        ((TIRImplObject) l.obj).doCommit(in, rep);
        l = l.next;
      }
      stKindImpl.toNormal();
    }
  }

  public void doAbort(long workId) throws RemoteException {
    LiItem l = firstEl;
    for(int i=0;i<numOfEl;i++) {
      ((TIRImplObject) l.obj).doAbort(workId);
      l = l.next;
    }
  }

  public BindTypeNormal add_bindtypenormal(String name) throws RemoteException, TIRExceptCreate {
    if (stKindImpl.value() == StateKind.sk_normal)
      throw new TIRExceptCreate("You can call add method on work objects only");
    BindTypeNormal ret = new BindTypeNormalImpl(new String(name));
    add(ret);
    return ret;
  }
  
  public BindTypeArray add_bindtypearray(String name, ExprOperDef[] indexes) throws RemoteException, TIRExceptCreate {
    if (stKindImpl.value() == StateKind.sk_normal)
      throw new TIRExceptCreate("You can call add method on work objects only");
    ExprOperDef[] np = new ExprOperDef [indexes.length];
    for (int i=0;i<indexes.length;i++) 
      np[i] = indexes[i];
    BindTypeArray ret = new BindTypeArrayImpl(new String(name), np);
    add(ret);
    return ret;
  }
}
