/* $Id: ExprBinOperationDefImpl.java,v 1.1.1.1 2003/02/11 16:19:41 bures Exp $ */
package SOFA.SOFAnode.Made.TIR.Impl;
import SOFA.SOFAnode.Made.TIR.*;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.io.IOException;

public class ExprBinOperationDefImpl extends UnicastRemoteObject implements ExprBinOperationDef, TIRImplObject {
  ExprOperKindImpl eok;
  DefinitionKindImpl defKindImpl;
  StateKindImpl stKindImpl;
  ExprOperDef oper1;
  ExprOperDef oper2;

  public ExprBinOperationDefImpl(int kind) throws RemoteException {
    eok = new ExprOperKindImpl(kind);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Expression);
    stKindImpl = new StateKindImpl(StateKind.sk_normal);
    oper1 = null;
    oper2 = null;
  }

  public ExprBinOperationDefImpl(int kind, ExprOperDef op1, ExprOperDef op2) throws RemoteException {
    eok = new ExprOperKindImpl(kind);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Expression);
    stKindImpl = new StateKindImpl(StateKind.sk_work);
    oper1 = op1;
    oper2 = op2;
  }

  public ExprBinOperationDefImpl() throws RemoteException {
    eok = null;
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Expression);
    stKindImpl = new StateKindImpl(StateKind.sk_normal);
    oper1 = null;
    oper2 = null;
  }

  /* from interface TIRObject */
  public DefinitionKind get_def_kind() throws RemoteException {
    return (DefinitionKind) defKindImpl;
  }

  /* from interface TIRObject */
  public StateKind get_state() throws RemoteException {
    return (StateKind) stKindImpl;
  }

  public ExprOperDef operand1() throws RemoteException {
    return oper1;
  }
  
  public ExprOperDef operand2() throws RemoteException {
    return oper2;
  }

  public ExprOperKind get_eok_kind() throws RemoteException {
    return (ExprOperKind) eok;
  }

  public ExprOperDef calculate() throws RemoteException, TIRExceptCalculate {
    ExprOperDef ret = null;
    ExprOperDef a = oper1.calculate();
    ExprOperDef b = oper2.calculate();
    switch (eok.value()) {
    case ExprOperKind.eok_plus:
      ret = ExprHelp.plus(a,b);
      break;
    case ExprOperKind.eok_minus:
      ret = ExprHelp.minus(a,b);
      break;
    case ExprOperKind.eok_mul:
      ret = ExprHelp.mul(a,b);
      break;
    case ExprOperKind.eok_div:
      ret = ExprHelp.div(a,b);
      break;
    case ExprOperKind.eok_mod:
      ret = ExprHelp.mod(a,b);
      break;
    case ExprOperKind.eok_or:
      ret = ExprHelp.or(a,b);
      break;
    case ExprOperKind.eok_xor:
      ret = ExprHelp.xor(a,b);
      break;
    case ExprOperKind.eok_and:
      ret = ExprHelp.and(a,b);
      break;
    case ExprOperKind.eok_shr:
      ret = ExprHelp.shr(a,b);
      break;
    case ExprOperKind.eok_shl:
      ret = ExprHelp.shl(a,b);
      break;
    default: throw new TIRExceptCalculate("Can't calculate.");
    }

    return ret;
  }

  public void save(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      st.curOutFile.writeInt(eok.value());
      ((TIRImplObject)oper1).save(st);
      ((TIRImplObject)oper2).save(st);
    } catch (IOException e) {
      throw new TIRExceptStorage("Can't write in directory "+st.current+".");
    }
  }
  
  public void load(Storage st) throws RemoteException, TIRExceptStorage {
    oper1 = Storage.readExprOperDef(st);
    oper2 = Storage.readExprOperDef(st);
  }

  public void postLoad(RepositoryImpl r) throws RemoteException, TIRExceptStorage {
    ((TIRImplObject) oper1).postLoad(r);
    ((TIRImplObject) oper2).postLoad(r);
  }

  public boolean isNew() {
    return (stKindImpl.value()==StateKind.sk_work);
  }

  public void canCommit() throws RemoteException, TIRExceptCommit {;}

  public void doCommit(Container in, Repository rep) throws RemoteException {
    if (stKindImpl.value()==StateKind.sk_work) {
      stKindImpl.toNormal();
      ((TIRImplObject)oper1).doCommit(in,rep);
      ((TIRImplObject)oper2).doCommit(in,rep);
    }
  }

  public void doAbort(long workId) throws RemoteException {}
}
