/* $Id: ProtocolUnOperationDefImpl.java,v 1.1.1.1 2003/02/11 16:19:41 bures Exp $ */
package SOFA.SOFAnode.Made.TIR.Impl;
import SOFA.SOFAnode.Made.TIR.*;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.io.IOException;

public class ProtocolUnOperationDefImpl extends UnicastRemoteObject implements ProtocolUnOperationDef, TIRImplObject, SProtocolOperDef {
  ProtocolOperKindImpl pok;
  DefinitionKindImpl defKindImpl;
  StateKindImpl stKindImpl;
  ProtocolOperDef oper;

  public ProtocolUnOperationDefImpl(int kind, ProtocolOperDef op) throws RemoteException {
    pok = new ProtocolOperKindImpl(kind);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_ProtocolOper);
    stKindImpl = new StateKindImpl(StateKind.sk_work);
    oper = op;
  }

  public ProtocolUnOperationDefImpl(int kind) throws RemoteException {
    pok = new ProtocolOperKindImpl(kind);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_ProtocolOper);
    stKindImpl = new StateKindImpl(StateKind.sk_normal);
    oper = null;
  }

  public ProtocolUnOperationDefImpl() throws RemoteException {
    pok = null;
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Protocol);
    stKindImpl = new StateKindImpl(StateKind.sk_normal);
    oper = null;
  }

  /* from interface TIRObject */
  public DefinitionKind get_def_kind() throws RemoteException {
    return (DefinitionKind) defKindImpl;
  }

  /* from interface TIRObject */
  public StateKind get_state() throws RemoteException {
    return (StateKind) stKindImpl;
  }

  public ProtocolOperDef operand() throws RemoteException {
    return oper;
  }

  public ProtocolOperKind get_po_kind() throws RemoteException {
    return pok;
  }

  public void save(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      st.curOutFile.writeInt(pok.value());
      Storage.writeProtocolOperDef(st, oper);
    } catch (IOException e) {
      throw new TIRExceptStorage("Can't write in directory "+st.current+".");
    }
  }

  public void load(Storage st) throws RemoteException, TIRExceptStorage {
    oper = Storage.readProtocolOperDef(st);
  }

  public void postLoad(RepositoryImpl r) throws RemoteException, TIRExceptStorage {
    ((TIRImplObject) oper).postLoad(r);
  }

  public boolean isNew() {
    return (stKindImpl.value()==StateKind.sk_work);
  }

  public void canCommit() throws RemoteException, TIRExceptCommit {;}

  public void doCommit(Container in, Repository rep) throws RemoteException {
    if (stKindImpl.value()==StateKind.sk_normal)
      return;
    stKindImpl.toNormal();
    ((TIRImplObject) oper).doCommit(in,rep);
  }

  public void doAbort(long workId) throws RemoteException {}

  public String to_text() throws RemoteException {
    StringBuffer sb = new StringBuffer("(");
    switch (pok.value()) {
    case ProtocolOperKind.pok_repetition:
      sb.append(((SProtocolOperDef)oper).to_text());
      sb.append(")*");
      break;
    default: break;  
    }
    return sb.toString();
  }
}
