/* $Id: ConnectorCacheNSImpl.java,v 1.1.1.1 2003/02/11 16:19:41 bures Exp $ */

/*
 * Connector cache name server.
 *
 * Created on January 8, 2003, 4:34 PM
 */

package SOFA.SOFAnode.Run.CCNS;

import java.io.*;
import java.rmi.*;
import java.rmi.server.*;
import java.rmi.registry.*;
import java.net.MalformedURLException;

/** Unicast remote object implementing remote interface.
 *
 * @author Tomas Bures
 * @version 1.0
 */
public class ConnectorCacheNSImpl extends java.rmi.server.UnicastRemoteObject implements ConnectorCacheNS {

    private java.util.HashMap byHash;
    private java.util.HashMap byName;
    
    private long counter=0;
    
    /** Constructs ConnectorCacheNSImpl object and exports it on default port.
     */
    public ConnectorCacheNSImpl() throws RemoteException {
        super();
        load();
    }
    
    /** Constructs ConnectorCacheNSImpl object and exports it on specified port.
     * @param port The port for exporting
     */
    public ConnectorCacheNSImpl(int port) throws RemoteException {
        super(port);
        load();
    }

    /** Register ConnectorCacheNSImpl object with the RMI registry.
     * @param name - name identifying the service in the RMI registry
     * @param create - create local registry if necessary
     * @throw RemoteException if cannot be exported or bound to RMI registry
     * @throw MalformedURLException if name cannot be used to construct a valid URL
     * @throw IllegalArgumentException if null passed as name
     */
    public static void registerToRegistry(String name, Remote obj, boolean create) throws RemoteException, MalformedURLException{
        
        if (name == null) throw new IllegalArgumentException("registration name can not be null");
        
        System.out.print("Registering as: "+name);
        try {
            Naming.rebind(name, obj);
        } catch (RemoteException ex){
            if (create) {
                Registry r = LocateRegistry.createRegistry(Registry.REGISTRY_PORT);
                r.rebind(name, obj);
            } else {
              System.out.println();
              throw ex;
            }
        }
        System.out.println("\rCCNS is bound on '"+name+"' and running.");
    }
    
    /** Main method.
     */
    public static void main(String[] args) {
        System.setSecurityManager(new RMISecurityManager());
        
        try {
            ConnectorCacheNSImpl obj = new ConnectorCacheNSImpl();

            String rmiport = System.getProperty("SOFA.CCNS.rmiport","1099");
            String rmihost = System.getProperty("SOFA.CCNS.rmihost","localhost");
                  
            registerToRegistry("//"+rmihost+":"+rmiport+"/CCNS", obj, false);
            
        } catch (RemoteException ex) {
            ex.printStackTrace();
        } catch (MalformedURLException ex) {
            ex.printStackTrace();
        }
    }
    
    private String createHash(String name) {
        counter++;
        String cStr=Long.toHexString(counter);
        return "H"+"0000000000000000".substring(0,16-cStr.length())+cStr;
    }

    private String getStorageFileName() {
        return System.getProperty("sofa.connector.ccns","ccns.dat");
    }

    private void store() {
        String fileName=getStorageFileName();
        
        try {
            ObjectOutputStream os=new ObjectOutputStream(new FileOutputStream(fileName));
            os.writeLong(counter);
            os.writeObject(byName);
            os.writeObject(byHash);
        } catch (Exception e) {
            System.err.println("Can't save the Connector Cache Name Server persistent storage.");
            e.printStackTrace();
        }
    }
    
    private void load() {
        String fileName=getStorageFileName();
        
        File inFile=new File(fileName);
        if (!inFile.canRead()) {
            byName=new java.util.HashMap();
            byHash=new java.util.HashMap();
            counter=0;
            System.err.println("Connector Cache Name Server persistent storage not found. Initializing a new one.");
            store();
        } else {
            try {
                ObjectInputStream is=new ObjectInputStream(new FileInputStream(inFile));
                counter=is.readLong();
                byName=(java.util.HashMap)is.readObject();
                byHash=(java.util.HashMap)is.readObject();
            } catch (Exception e) {
                System.err.println("Can't read the Connector Cache Name Server persistent storage.");
                e.printStackTrace();
            }
        }
    }
    
    /** Returns long connector name for the short unique tag. If the entry is not present in registry, <code>null</code> is returned.
     * @param name Unique short tag
     * @throws RemoteException
     * @return Long connector name
     */    
    public synchronized String getHashByName(java.lang.String name) throws RemoteException {
        String hash;
        if (!byName.containsKey(name)) {
            hash=createHash(name);
            byName.put(name,hash);
            byHash.put(hash,name);
            store();
        } else {
            hash=(String)byName.get(name);
        }
        return hash;
    }
    
    /** Returns long connector name for the short unique tag. If the entry is not present in registry, <code>null</code> is returned.
     * @param name Unique short tag
     * @throws RemoteException
     * @return Long connector name
     */    
    public synchronized String getNameByHash(java.lang.String hash) throws RemoteException {
        String name;
        if (!byHash.containsKey(hash)) {
            name=null;
        } else {
            name=(String)byHash.get(hash);
        }
        return name;
    }
    
}
