/* $Id: KillApl.java,v 1.1.1.1 2003/02/11 16:19:41 bures Exp $ */
package SOFA.SOFAnode.Run;

import SOFA.SOFAnode.Run.DeplDockRegistry.*;
import SOFA.SOFAnode.Run.Deployment.DeplDock;

import org.omg.CosNaming.*;

/** Stops a SOFA application.
  *
  * @author Petr Hnetynka
  */
public class KillApl {
  /** Stops a SOFA application. 
    * Arguments: 1st - (required) name of the depldock, 2nd - (optional) name of the component
    */
  public static void main(String[] argv) {
    if ((argv.length < 1) && (argv.length > 2)) {
      System.out.println("Bad number of arguments.");
      System.exit(1);
    }
    if (System.getSecurityManager() == null) {
      System.setSecurityManager(new java.rmi.RMISecurityManager());
    }

    SOFA.Util.CORBAAccess ca = new SOFA.Util.CORBAAccess();

    NameComponent nc = new NameComponent("DeplDockRegistry", "");
    NameComponent path[] = {nc};
    DeplDockRegistry ddr = null;
    try {
      System.out.print("Obtaining depldock registry...");
      ddr = DeplDockRegistryHelper.narrow(SOFA.Util.CORBAAccess.ncRef.resolve(path));
      System.out.println("OK");
    } catch (Exception e) {
      System.out.println("Can't get DeplDock registry.");
      e.printStackTrace();
      System.exit(1);
    }
    
    
    byte[] serRef = null;
    try {
      System.out.print("Obtaining reference to depldock...");
      serRef = ddr.lookup(argv[0]);
      System.out.println("OK");
    } catch (NotFoundException e) {
      System.out.println("No deployment dock with name \""+argv[1]+"\".");
      System.exit(1);
    }
    
    SOFA.Connector.Reference ddRef = SOFA.SOFAnode.Run.Deployment.DeplDockImpl.bytesToSofaReference(serRef);
   
    DeplDock ddock = null;
    try {
      System.out.print("Creating connector to depldock...");
      ddock = (DeplDock) SOFA.Connector.Boot.DeplDockConnector.createClt(ddRef);
      System.out.println("OK");
    } catch (SOFA.Connector.ConnectorException e) {
      System.out.println("Can't create connector to deployment dock.");
      System.exit(1);
    }

    String nameOK = null;
    String[] names = ddock.namesOfAllRunningApplications();
    
    if (argv.length == 2) {
      boolean found = false;
      for (int i=0; i< names.length; i++) {
        if (names[i].compareTo(argv[1])==0) {
          found = true;
          break;
        }
      }
      if (!found) {
        System.out.println("No application with given name");
        System.exit(1);
      }
      nameOK = argv[1];
    } else {
      System.out.println("Choose application to destroy");
      System.out.println("=============================");
      for (int i=0;i<names.length; i++) {
        System.out.print(i);
        System.out.print("...");
        System.out.println(names[i]);
      }
      try {
        java.io.BufferedReader in = new java.io.BufferedReader(new java.io.InputStreamReader(System.in));
        String ln = in.readLine();
        ln.trim();
        int num = Integer.parseInt(ln);
        if (num<0 || num>=names.length) {
          System.out.println("Bad number");
          System.exit(1);
        }
        nameOK = names[num];
      } catch (Exception e) {
        System.out.println("Exception: "+e.getMessage());
        e.printStackTrace();
        System.exit(1);
      }
    }

    SOFA.Connector.Reference apl = null;
    try {
      System.out.print("Obtaining reference to CM...");
      apl = ddock.getReference(nameOK);
      System.out.println("OK");
    } catch (SOFA.SOFAnode.Run.Deployment.DeploymentException e) {
      System.out.println("DeploymentException: "+e.getMessage());
      e.printStackTrace();
      System.exit(1);
    }

    SOFA.Component.DCUP.DCUPComponentManager aplcm = null;
    try {
      System.out.print("Creating connector to CM...");
      aplcm = (SOFA.Component.DCUP.DCUPComponentManager) SOFA.Connector.Boot.DCUPComponentManagerConnector.createClt(apl);
      System.out.println("OK");
    } catch (SOFA.Connector.ConnectorException e) {
      System.out.println("Can't create connector to CM.");
      System.exit(1);
    }

    try {
      System.out.print("Destroying component...");
      aplcm.destroyComponent(new SOFA.SOFAnode.Run.StorageImpl());
      System.out.println("OK");
    } catch (SOFA.Component.ComponentLifecycleException e) {
      System.out.println("ComponentLifecycleException: "+e.getMessage());
      e.printStackTrace();
      System.exit(1);
    }
  }
}
