/* $Id: BehaviorProtocolChecker.java,v 1.1.1.1 2003/02/11 16:19:41 bures Exp $ */
package SOFA.SOFAnode.Util;

/**
 * An interface for behavior protocol conformance checker. It allows to check:
 * interface protocol vs frame protocol (both provides and requires)
 * frame protocol vs architecture protocol
 *
 * @author  Stanislav Visnovsky
 * @version 1.0.0
*/

public interface BehaviorProtocolChecker {

/**
 * Tests a protocol conformance of interface protocol to frame protocol for both
 * provides and requires interfaces. All parameters have to share common namespace,
 * no renaming takes place.
 *
 * @author      Petr Hnetynka
 * @version     1.0.0
 * @return      a string message with error code or null for success
 * @parm ifaceProt       interface protocol
 * @parm frameProt       frame protocol
 * @parm ifaceOperations array of fully qualified interface operation names of the interface
 * @parm provide         whether it is provides (true) or requires interface (false)
*/
  public String int2frame(String ifaceProt, boolean provide, String[] ifaceOperations, String frameProt)
    throws SyntaxErrorException;

/**
 * Tests a protocol conformance of frame protocol to architecture protocol.
 * All parameters have to share common namespace, no renaming takes place.
 *
 * @author      Stanislav Visnovsky
 * @version     1.0.0
 * @return      a string message with error code or null for success
 * @parm frameProt            frame protocol
 * @parm archProt             architecture protocol
 * @parm providesOperations   array of fully qualified operation names on all provides interfaces of the frame
 * @parm ifaceOperations      array of fully qualified operation names on all interfaces of the frame
*/
  public String frame2arch(String frameProt, String[] providesOperations, String archProt, String[] ifaceOperations)
    throws SyntaxErrorException;

/**
 * Allows to get an information about maximal number of states reached through the last test.
 *
 * @author      Stanislav Visnovsky
 * @version     1.0.0
 * @return      a maximal number of states created
*/
  public int StatisticsMaxStates();
}
