/* $Id: SOFAURI.java,v 1.2 2003/03/17 11:18:12 hnetynka Exp $ */
package SOFA.Util;

import SOFA.SOFAnode.Run.SOFARuntimeException;

import java.net.MalformedURLException;
import java.net.URL;
import java.net.InetAddress;
import java.net.UnknownHostException;

import SOFA.SOFAnode.Run.DeplDockRegistry.*;

import org.omg.CosNaming.*;



/** Methods for manipulation with SOFA URI.
  *
  * @author Petr Hnetynka
  */
public class SOFAURI {

  /** Exception for bad SOFA URI */
  public static class BadSOFAURIException extends cz.cuni.sofa.lib.SystemException {
    public BadSOFAURIException() {
      super();
    }
  
    public BadSOFAURIException(String __reason) {
      super(__reason);
    }

    public BadSOFAURIException(String __reason, Throwable __cause) {
      super(__reason, __cause);
    }
  }

   /** Returns serialized reference to depldock from sofa URI. SOFA URI is in the form sofa://name_of_sofa_node/name_of_depldock
    * @param sofaURI  SOFA URI
    * @return serialized form of sofa reference
    * @throws BadSOFAURIException   bad URI
    */
  public static byte[] getReferenceToDeplDockFromSOFAURI(String sofaURI) throws BadSOFAURIException {
    java.net.URI uri = null;
    try {
      uri = new java.net.URI(sofaURI);
    } catch (java.net.URISyntaxException e) {
      throw new BadSOFAURIException("URISynaxException: "+e.getMessage(), e);
    }
    if (uri.getScheme().compareTo("sofa") != 0)
      throw new BadSOFAURIException("Bad scheme of the URI");
    String sofaNodeName = System.getProperty("sofa.nodename","");
    if (sofaNodeName.compareTo("")==0) {
      try {
        sofaNodeName = InetAddress.getLocalHost().getHostName();
      } catch (UnknownHostException e) {}
    }
    String host = uri.getHost();
    if (host.compareTo("local")==0 || host.compareTo(sofaNodeName)==0)
      throw new BadSOFAURIException("Can't locate sofa node");
    String dockName = uri.getPath();
    if (dockName.lastIndexOf('/')!=0)
      throw new BadSOFAURIException("Bad name of the depldock");
    dockName = dockName.substring(1);
    
    SOFA.Util.CORBAAccess ca = new SOFA.Util.CORBAAccess();
    NameComponent nc = new NameComponent("DeplDockRegistry", "");
    NameComponent path[] = {nc};
    DeplDockRegistry ddr = null;
    try {
      ddr = DeplDockRegistryHelper.narrow(SOFA.Util.CORBAAccess.ncRef.resolve(path));
    } catch (Exception e) {
      throw new SOFA.SOFAnode.Run.SOFARuntimeException("Can't get DeplDock registry.");
    }
    
    byte[] serRef = null;
    try {
      serRef = ddr.lookup(dockName);
    } catch (NotFoundException e) {
      throw new BadSOFAURIException("No deployment dock with name \""+dockName+"\".");
    }
    return serRef;
  }

  /** Tests if URI is local.
    * @param uri    SOFA URI
    * @return       true if URI is local
    */
  public static boolean isLocal(java.net.URI uri) {
    if (uri.getScheme().compareTo("sofa") != 0)
      return false;
    if (uri.getHost().compareTo("local")!=0)
      return false;
    if (uri.getPath().compareTo("/local")!=0)
      return false;
    return true;
  }

  /** Tests if URI is local.
    * @param uri    SOFA URI
    * @return       true if URI is local
    * @throws BadSOFAURIException  bad URI
    */
  public static boolean isLocal(String uri) throws BadSOFAURIException {
    java.net.URI u = null;
    try {
      u = new java.net.URI(uri);
    } catch (java.net.URISyntaxException e) {
      throw new BadSOFAURIException("URISynaxException: "+e.getMessage(), e);
    }
    return isLocal(u);
  }
 
}
